"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClockCodec = void 0;
const clock_1 = require("../../../json-crdt-patch/clock");
class ClockCodec {
    constructor(clock, table1) {
        this.clock = clock;
        this.table2 = new Map();
        this.table1 = table1;
    }
    encode() {
        const { clock } = this;
        let str = '[' + clock.sessionId + ',' + clock.time;
        this.table2.set(clock.sessionId, 1);
        let i = 2;
        for (const c of clock.clocks.values()) {
            if (c.sessionId !== clock.sessionId) {
                str += ',' + c.sessionId + ',' + c.time;
                this.table2.set(c.sessionId, i++);
            }
        }
        return str + ']';
    }
    static decode(data) {
        const table1 = new Map();
        const [sessionId, time] = data;
        const clock = new clock_1.VectorClock(sessionId, time);
        table1.set(1, clock);
        const length = data.length;
        let j = 2;
        let i = 2;
        while (i < length) {
            const sessionId = data[i++];
            const time = data[i++];
            const timestamp = new clock_1.LogicalTimestamp(sessionId, time);
            table1.set(j++, timestamp);
            clock.observe(timestamp, 1);
        }
        return new ClockCodec(clock, table1);
    }
    encodeTs(ts) {
        const { sessionId, time } = ts;
        const clockId = this.table2.get(sessionId);
        if (!clockId)
            return sessionId + ',' + time;
        const clock = this.clock.clocks.get(sessionId);
        return clockId + ',' + (clock.time - time);
    }
    decodeTs(clockId, time) {
        const clock = this.table1.get(clockId);
        if (!clock)
            return new clock_1.LogicalTimestamp(clockId, time);
        return new clock_1.LogicalTimestamp(clock.sessionId, clock.time - time);
    }
}
exports.ClockCodec = ClockCodec;
