"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectType = void 0;
var tslib_1 = require("tslib");
var constants_1 = require("../../../json-crdt-patch/constants");
var ObjectEntry_1 = require("./ObjectEntry");
var json_schema_serializer_1 = require("json-schema-serializer");
var constants_2 = require("../../constants");
var decodeNode_1 = require("../../codec/compact/decodeNode");
var ObjectType = (function () {
    function ObjectType(doc, id) {
        this.doc = doc;
        this.id = id;
        this.latest = new Map();
    }
    ObjectType.prototype.get = function (key) {
        var entry = this.latest.get(key);
        return entry ? entry.value : undefined;
    };
    ObjectType.prototype.getId = function (key) {
        var entry = this.latest.get(key);
        return entry ? entry.id : undefined;
    };
    ObjectType.prototype.insert = function (op) {
        var e_1, _a;
        var last = this.latest;
        var clock = op.id.clock();
        try {
            for (var _b = tslib_1.__values(op.tuples), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], value = _d[1];
                var id = clock.tick(1);
                var lastEntry = last.get(key);
                if (lastEntry)
                    if (lastEntry.id.compare(id) >= 0)
                        continue;
                this.put(key, id, value);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    ObjectType.prototype.put = function (key, id, value) {
        var entry = new ObjectEntry_1.ObjectEntry(id, value);
        this.latest.set(key, entry);
    };
    ObjectType.prototype.toJson = function () {
        var e_2, _a;
        var obj = {};
        try {
            for (var _b = tslib_1.__values(this.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], entry = _d[1];
                var value = entry.value;
                if (value.compare(constants_1.UNDEFINED_ID) === 0)
                    continue;
                var node = this.doc.nodes.get(value);
                if (!node)
                    continue;
                obj[key] = node.toJson();
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return obj;
    };
    ObjectType.prototype.toString = function (tab) {
        var e_3, _a;
        if (tab === void 0) { tab = ''; }
        var str = tab + "ObjectType(" + this.id.toDisplayString() + ")";
        try {
            for (var _b = tslib_1.__values(this.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], value = _d[1];
                var node = this.doc.nodes.get(value.value) || constants_2.UNDEFINED;
                str += "\n" + tab + "  \"" + key + "\" :\n" + node.toString(tab + '    ');
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return str;
    };
    ObjectType.prototype.clone = function (doc) {
        var e_4, _a;
        var obj = new ObjectType(doc, this.id);
        try {
            for (var _b = tslib_1.__values(this.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], _e = _d[1], id = _e.id, value = _e.value;
                obj.put(key, id, value);
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return obj;
    };
    ObjectType.prototype.children = function () {
        var _a, _b, value, e_5_1;
        var e_5, _c;
        return tslib_1.__generator(this, function (_d) {
            switch (_d.label) {
                case 0:
                    _d.trys.push([0, 5, 6, 7]);
                    _a = tslib_1.__values(this.latest.values()), _b = _a.next();
                    _d.label = 1;
                case 1:
                    if (!!_b.done) return [3, 4];
                    value = _b.value.value;
                    return [4, value];
                case 2:
                    _d.sent();
                    _d.label = 3;
                case 3:
                    _b = _a.next();
                    return [3, 1];
                case 4: return [3, 7];
                case 5:
                    e_5_1 = _d.sent();
                    e_5 = { error: e_5_1 };
                    return [3, 7];
                case 6:
                    try {
                        if (_b && !_b.done && (_c = _a.return)) _c.call(_a);
                    }
                    finally { if (e_5) throw e_5.error; }
                    return [7];
                case 7: return [2];
            }
        });
    };
    ObjectType.prototype.encodeCompact = function (codec) {
        var e_6, _a;
        var str = '[0,' + codec.encodeTs(this.id);
        try {
            for (var _b = tslib_1.__values(this.latest.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
                var _d = tslib_1.__read(_c.value, 2), key = _d[0], value = _d[1];
                var node = this.doc.nodes.get(value.value);
                str += ',' + json_schema_serializer_1.asString(key) + ',' + codec.encodeTs(value.id) + ',' + node.encodeCompact(codec);
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_6) throw e_6.error; }
        }
        return str + ']';
    };
    ObjectType.decodeCompact = function (doc, codec, data) {
        var length = data.length;
        var objId = codec.decodeTs(data[1], data[2]);
        var obj = new ObjectType(doc, objId);
        for (var i = 3; i < length;) {
            var key = data[i++];
            var id = codec.decodeTs(data[i++], data[i++]);
            var value = decodeNode_1.decodeNode(doc, codec, data[i++]);
            obj.put(key, id, value.id);
        }
        return obj;
    };
    return ObjectType;
}());
exports.ObjectType = ObjectType;
